;;; rtca-defun.el --- rtca defun list
;;;$Id: rtca-defun.el,v 1.17 2002/04/20 01:13:08 wackie Exp $	

;; Copyright (C) 2002 by Takashi KIMURA

;; Author: Takashi KIMURA <wackie@mbb.nifty.ne.jp>
;; Keywords: network, NIFTY, RTC

;; This file is free software; you can redistribute it and/or modify
;; it under the terms of the GNU General Public License as published by
;; the Free Software Foundation; either version 2, or (at your option)
;; any later version.

;; This file is distributed in the hope that it will be useful,
;; but WITHOUT ANY WARRANTY; without even the implied warranty of
;; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;; GNU General Public License for more details.

;; You should have received a copy of the GNU General Public License
;; along with GNU Emacs; see the file COPYING.  If not, write to
;; the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
;; Boston, MA 02111-1307, USA.

;;; Code:
(provide 'rtca-defun)

(defun rtca-matching-substiring (string arg)
  (substring string (match-beginning arg) (match-end arg)))

(defun rtca-next-line (&optional n)
  (interactive)
  (if (= (point) (point-max))
      (newline))
  (forward-line n))

;;;
;;; Output window
;;;
(defun rtca-recenter-point-max ()
  (or (pos-visible-in-window-p (point-max) (selected-window))
      (recenter nil)))

;;; Output id window
(defun rtca-output-id-handle (id handle)
  (let (string x)
    (setq string (concat "  "id ":" handle rtca-next-line))
    (if (setq x (cdr (assoc id rtca-handle-list)))
	(unless (equal x handle)
	  (rtca-add-handle-list id handle)
	  (rtca-unique-handle))
      (progn
	(rtca-add-handle-list id handle)
	(rtca-output-id string)))
    ))

(defun rtca-add-id-buffer (string)
  (save-selected-window 
    (select-window rtca-id-window)
    (rtca-add-string-to-buffer rtca-id-buffer string)
    ))

(defun rtca-output-id (string)
  (save-selected-window 
    (select-window rtca-id-window)
    (rtca-add-string-to-buffer-top rtca-id-buffer string)
    (recenter 1)
    ))

(defun rtca-add-handle-list (id handle)
  (setq rtca-handle-list (cons (cons id handle) rtca-handle-list))
  )

(defun rtca-unique-handle ()
  (interactive)
  (let (id handle)
    (setq rtca-tmp-list '())
    (save-selected-window 
      (select-window rtca-id-window)
      (save-current-buffer
	(set-buffer rtca-id-buffer)
	(erase-buffer)
	(save-current-buffer
	  (while (> (safe-length rtca-handle-list) 0)
	    (progn
	      (setq id (caar rtca-handle-list))
	      (setq handle (cdar rtca-handle-list))
	      (setq rtca-handle-list (cdr rtca-handle-list))
	      (unless (assoc id rtca-tmp-list)
		(setq rtca-tmp-list (cons (cons id handle) rtca-tmp-list))
		(rtca-add-id-buffer 
		 (concat "  "id ":" handle rtca-next-line))
		)))
	  (setq rtca-handle-list (nreverse rtca-tmp-list)))
	(goto-char (point-min))
	(recenter 1)
      ))))

;;; Output rt window
(defun rtca-output-rt-chat (chan id handl line format)
  (let (string)
    (cond 
     ((= format 1) (setq string (concat "(" handl ")" line )))
     ((= format 2) (setq string (concat "(" id ")"    line )))
     ((= format 3) (setq string (concat "(" id "," handl ")" line)))
     ((= format 4) (setq string (concat "(" chan "," handl ")" line )))
     ((= format 5) (setq string (concat "(" chan "," id ")"    line )))
     (t (setq string (concat "(" chan "," id "," handl ")" line))))
    (rtca-output-rt string)
    ))

(defun rtca-output-rt (string)
  (save-selected-window 
    (select-window rtca-rt-window)
    (rtca-add-string-to-buffer rtca-rt-buffer string)
    (rtca-next-line 1)
    (rtca-recenter-point-max)
    ))

;;;Connect to NIFTY
(defun rtca-connect-nif ()
  (interactive)
  (unless (get-buffer-process rtca-process-buffer)
    (setq rtca-process 
	  (start-process "RTCA" rtca-process-buffer rtca-program )))
  (set-process-coding-system rtca-process 'sjis-dos 'sjis-dos)
  (set-process-filter rtca-process 'rtca-initial-filter)
  (accept-process-output rtca-process)
  (erase-buffer)
  (process-send-string rtca-process (concat "open " rtca-host rtca-new-line)))

(defun rtca-send-initial-command (proc)
  (interactive)
  (process-send-string proc (concat "/ID ON" rtca-new-line ))
  (if (equal rtca-handle "") nil
    (process-send-string proc (concat "/ha " rtca-handle rtca-new-line )))
  )

(defun rtca-auto-goto-rtc ()
  (interactive)
  (if (equal rtca-default-forum "") nil
    (setq rtca-forum rtca-default-forum)
    (rtca-goto-rtc rtca-forum rtca-channel))
  )

(defun rtca-manual-goto-rtc ()
  (interactive)
  (setq rtca-forum
	(read-string "FORUM: " rtca-forum))
  (setq rtca-channel
	(read-string "CHANNEL: " rtca-channel nil ""))
  (rtca-goto-rtc rtca-forum rtca-channel))
  

(defun rtca-goto-rtc (forum channel)
  (interactive)
  (rtca-goto-rtca-input-window)
  (if (equal rtca-status "OFF") (error "NOT Connected to NIFTY")
      (progn 
	(set-process-filter rtca-process 'rtca-goto-rtc-filter)
	(if (equal rtca-status "RTC")
	    (process-send-string rtca-process 
				 (concat "/go " rtca-forum rtca-new-line ))
	  (process-send-string rtca-process 
			       (concat "go " rtca-forum rtca-new-line )))
	)))

(defun rtca-goto-rtc-filter (proc string)
  (save-excursion
    (let* ((case-fold-search t))
      (cond
       ((string-match "^FORUM>" string)
	(rtca-filter proc string)
	(setq rtca-status "NIF")
	(process-send-string proc (concat "rtc" rtca-new-line)))
       
       ((string-match "^$B"!HV9f(B" string)
	(rtca-filter proc string)
	(set-process-filter proc 'rtca-filter)
	(error (concat "NO SUCH FORUM as :" (upcase rtca-forum))))

       ((string-match "^$B!'(B" string)
	(rtca-filter proc string)
	(if (equal rtca-channel "")
	      (setq rtca-channel
		    (read-string "CHANNEL: " "" nil "1"))
	      )
	(process-send-string proc (concat rtca-channel rtca-new-line )))

       ((string-match "^$B!]%j%"%k%?%$%`(B" string)
	(rtca-filter proc string)
	(rtca-show-status 
	 (concat (upcase rtca-forum) " CH (" rtca-channel ") "))
	(set-process-filter proc 'rtca-filter))
       (t (rtca-filter proc string))
       ))))
;;;
;;; end command
;;;
(defun rtca-end ()
  (interactive)
  (if (y-or-n-p "really exit RTCA?")
      (progn
	(rtca-goto-rtca-input-window)
	(set-process-filter rtca-process 'rtca-end-filter)
	(unless (equal rtca-status "OFF")
	  (rtca-disconnect-nif))
	(delete-windows-on rtca-input-buffer)
	(delete-windows-on rtca-log-buffer)
	(delete-windows-on rtca-rt-buffer)
	(delete-windows-on rtca-id-buffer)
	(delete-windows-on rtca-process-buffer)
	(kill-buffer rtca-input-buffer)
	(kill-buffer rtca-log-buffer)
	(kill-buffer rtca-rt-buffer)
	(kill-buffer rtca-id-buffer)
	(kill-buffer rtca-process-buffer)
	(run-hooks 'rtca-exit-hook)
	)))

(defun rtca-disconnect-nif ()
  (interactive)
  (if (equal rtca-status "RTC")
      (process-send-string rtca-process 
			   (concat "/e " rtca-new-line )))
  (process-send-string rtca-process (concat "bye" rtca-new-line ))
  )

(defun rtca-end-filter (proc string)
  (interactive)
  (save-excursion
    (cond 
     ((string-match "^FORUM>" string)
      (rtca-filter proc string)
      (process-send-string rtca-process (concat "bye" rtca-new-line )))

     ((string-match "^Connection closed" string)
      (rtca-filter proc string)
      (set-process-filter rtca-process 'rtca-filter)
      ))))
;;;
;;; filter
;;;
(defun rtca-initial-filter (proc string)
  (save-current-buffer
    (set-buffer (process-buffer proc))
    (let* ((case-fold-search t))
      (cond ((string-match "No such host" string)
	     (kill-buffer (process-buffer proc))
	     (error "No such host"))
	    ((string-match "passw" string)
	     (rtca-filter proc string)
	     (send-string proc (concat (read-passwd "Password: ")
				       rtca-new-line))
	     (clear-this-command-keys))
	    ((string-match "Connection-ID" string)
	     (rtca-filter proc string)
	     (send-string proc (concat "SVC" rtca-new-line)))
	    ((string-match "User-ID" string)
	     (rtca-filter proc string)
	     (send-string proc 
			  (concat 
			   (setq rtca-usr-id 
				 (read-no-blanks-input "User-ID: " rtca-usr-id))
			   rtca-new-line)))
	    (t (rtca-filter proc string)
	       (when (string-match "NIFTY SERVE" string)
		 (rtca-filter proc string)
		 (rtca-show-status "NIFTY")
		 (setq rtca-status "NIF")
		 (if (equal rtca-auto-goto nil)
		     (set-process-filter proc 'rtca-filter)
		   (rtca-auto-goto-rtc))
		 ))
	    ))))

(defun rtca-filter (proc string)
  (interactive)
  (save-excursion
    ;;write to process buffer
    (rtca-add-string-to-buffer rtca-process-buffer string)
    
    ;;write to rt buffer
    (cond 
     ((string-match "^(\\([12]\\),\\([A-Z][A-Z][A-Z][0-9][0-9][0-9][0-9][0-9]\\),\\([^)]+\\))\\(.*\\)" string)
      (setq channel (rtca-matching-substiring string 1)
	    id (rtca-matching-substiring string 2)
	    handle (rtca-matching-substiring string 3)
	    line (rtca-matching-substiring string 4)
	    )
      (rtca-output-rt-chat channel id handle line rtca-format)
      (rtca-output-id-handle id handle))
     
     ((string-match "^(\\([12]\\),\\([^)]+\\))\\(.*\\)" string)
      (setq channel (rtca-matching-substiring string 1)
	    id       rtca-id-unknown
	    handle  (rtca-matching-substiring string 2)
	    line    (rtca-matching-substiring string 3)
	    )
      (rtca-output-rt-chat channel id handle line rtca-format)
      (rtca-output-id-handle id handle)
      (process-send-string rtca-process (concat "/ID ON" rtca-new-line )))
     
     ((string-match "^$B!]%j%"%k%?%$%`(B" string)
      (setq rtca-status "RTC")
      (rtca-output-rt string)
      (rtca-send-initial-command rtca-process)
      )

     ((string-match "^FORUM>" string)
      (rtca-show-status "NIFTY")
      (setq rtca-status "NIF")
      (rtca-add-log-buffer string))

     ((string-match "^Connection closed" string)
      (rtca-show-status "OFF")
      (setq rtca-status "OFF")
      (rtca-add-log-buffer string))

     (t (rtca-add-log-buffer string))
     )))

(defun rtca-show-status (string)
  (save-selected-window 
    (select-window rtca-rt-window)
    (setq string (concat "RTCA:[ " string " ]"))
    (if (get-buffer string)
	(set-window-buffer rtca-rt-window string)
	(rename-buffer string t)
	)))

(defun rtca-add-log-buffer (string)
  (save-selected-window 
    (select-window rtca-log-window)
    (rtca-add-string-to-buffer rtca-log-buffer string)
    (rtca-recenter-point-max)
    ))

(defun rtca-add-string-to-buffer (buffer string)
    (set-buffer buffer)
    (goto-char (point-max))
    (insert-before-markers string) )

(defun rtca-add-string-to-buffer-top (buffer string)
    (set-buffer buffer)
    (goto-char (point-min))
    (insert-before-markers string) )

(defun rtca-insert-string-to-buffer (buffer string)
    (set-buffer buffer)
    (insert-before-markers string))

;;;
;;; input command
;;;
(defun rtca-send-line ()
  "Send the current line to the rtca process."
  (interactive)
  (beginning-of-line)
  (process-send-string rtca-process
		       (concat
			(buffer-substring (point)
					  (progn (end-of-line) (point)))
			rtca-new-line))
  (rtca-next-line 1))

(defun rtca-send-region ()
  "Send the region to the rtca process ."
  (interactive)
  (process-send-string rtca-process
		       (concat
			(buffer-substring (region-beginning)
					  (region-end))
			rtca-new-line)))

(defun rtca-skk-send-line ()
  "Send the current line to the rtca process with SKK mode considered."
  (interactive)
  (cond ((not (boundp 'skk-mode))
	 (rtca-send-line))
	((not skk-mode)
	 (rtca-send-line))
	((not skk-henkan-on)
	 (rtca-send-line))
	(skk-egg-like-newline
	 (skk-kakutei))
	(t
	 (skk-kakutei)
	 (rtca-send-line))))

(defun rtca-quote-line-and-goto-input-close-history ()
  (interactive)
  (rtca-quote-line-to-input)
  (rtca-goto-rtca-input-window)
  (rtca-close-history-window)
  )

(defun rtca-quote-line-and-goto-input ()
  (interactive)
  (rtca-quote-line-to-input)
  (rtca-goto-rtca-input-window)
  )

(defun rtca-quote-line-to-input ()
  "Quote the current line to the input buffer."
  (interactive)
  (save-excursion
    (beginning-of-line)
    (save-current-buffer
      (rtca-insert-string-to-buffer 
       rtca-input-buffer
       (concat rtca-quote-mark
	       (buffer-substring (point)
				 (progn (end-of-line) (point))))
       ))))


(defun rtca-send-command ()
  (interactive)
 (rtca-id-command "/send")) 

(defun rtca-profile-command ()
  (interactive)
 (rtca-id-command "/prof")) 

(defun rtca-id-command (command)
  (interactive)
  (beginning-of-line)
  (setq string 
	(buffer-substring (point) (progn (end-of-line) (point))))
  (string-match
   "\\([A-Z][A-Z][A-Z][0-9][0-9][0-9][0-9][0-9]\\):\\(.*\\)$" string)
  (rtca-insert-string-to-buffer 
   rtca-input-buffer
   (concat command " " (rtca-matching-substiring string 1) " "))
  (rtca-goto-rtca-input-window)
  )

;;;
;;; window managemant
;;;

(defun rtca-initial-open-window ()
  (interactive)
  (rtca-open-window)
  (rtca-show-status " OFF ")
  )

(defun rtca-open-window ()
  (interactive)
  ;;initial open window
  (setq rtca-input-window 
	(get-buffer-window
	 (setq rtca-input-buffer  (pop-to-buffer "*rtca-input*"))
	 ))
  (rtca-input-mode)
  (save-current-buffer
    (delete-other-windows)
    (set-window-buffer 
     (setq rtca-rt-window 
	   (split-window rtca-input-window rtca-input-window-height))
    (set-buffer 
     (setq rtca-rt-buffer (get-buffer-create "*rtca-rt*"))))
    (rtca-log-mode)
    (set-window-buffer 
     (setq rtca-log-window (split-window rtca-rt-window 
					 (- (window-height rtca-rt-window) 10))
	   )
     (set-buffer 
      (setq rtca-log-buffer (get-buffer-create "*rtca-log*"))))
    (rtca-log-mode)
    (set-window-buffer 
     (setq rtca-id-window (split-window rtca-log-window nil t))
     (set-buffer 
      (setq rtca-id-buffer (get-buffer-create "*rtca-id*"))))
    (rtca-id-mode)
    (set-buffer 
     (setq rtca-process-buffer (get-buffer-create "*rtca*")))
    (rtca-mode)
    ))

(defun rtca-open-history-window ()
  (interactive)
    (save-selected-window (rtca-make-history-window)))
  
(defun rtca-close-history-window ()
  (interactive)
  (if (window-live-p rtca-history-window)
      (progn
	(delete-window rtca-history-window)
	(rtca-goto-rtca-input-window))))

(defun rtca-goto-rtca-input-window ()
  (interactive)
  (select-window rtca-input-window)
  (rtca-recenter-point-max)
  )

(defun rtca-goto-rtca-rt-window ()
  (interactive)
  (select-window rtca-rt-window)
  )

(defun rtca-goto-rtca-id-window ()
  (interactive)
  (select-window rtca-id-window)
  )

(defun rtca-goto-rtca-history-window ()
  (interactive)
  (rtca-make-history-window)
  (select-window rtca-history-window))

(defun rtca-make-history-window ()
  (interactive)
  (let (string)
    (setq string "HISTORY")
    (if (and (windowp rtca-history-window)
	     (window-live-p rtca-history-window))
	nil
      (progn
	(unless (window-live-p rtca-rt-window)
	  (rtca-open-window))
	(setq rtca-history-window (split-window rtca-rt-window))))
    (unless (get-buffer string)
      (make-indirect-buffer (window-buffer rtca-rt-window) string))
    (set-window-buffer rtca-history-window string)
    (save-current-buffer
      (set-buffer string)
      (rtca-history-mode))
    ))
  
(defun rtca-clear-buffer (buffer)
  (save-current-buffer
    (set-buffer buffer)
    (erase-buffer)
    ))
  

(run-hooks 'rtca-defun-load-hook)
;;; rtca-defun.el ends here
