;;; rtca-misc.el --- rtca misc defun list
;;;	$Id: rtca-misc.el,v 1.7 2002/04/20 02:04:22 wackie Exp $	

;; Copyright (C) 2002 by Takashi KIMURA

;; Author: Takashi KIMURA <wackie@mbb.nifty.ne.jp>
;; Keywords: network, NIFTY, RTC

;; This file is free software; you can redistribute it and/or modify
;; it under the terms of the GNU General Public License as published by
;; the Free Software Foundation; either version 2, or (at your option)
;; any later version.

;; This file is distributed in the hope that it will be useful,
;; but WITHOUT ANY WARRANTY; without even the implied warranty of
;; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;; GNU General Public License for more details.

;; You should have received a copy of the GNU General Public License
;; along with GNU Emacs; see the file COPYING.  If not, write to
;; the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
;; Boston, MA 02111-1307, USA.

;;; Code:
(provide 'rtca-misc)

;;;
;;; quote handle
;;;
(defun rtca-quote-handle-and-goto-input ()
  (interactive)
  (rtca-quote-handle-input)
  (rtca-goto-rtca-input-window)
  )

(defun rtca-quote-handle-input ()
  "Quote the handle to the input buffer."
  (interactive)
  (rtca-insert-string-to-buffer rtca-input-buffer (rtca-quote-handle)))

(defun rtca-quote-handle ()
  (interactive)
  (let (string)
    (save-excursion
      (save-current-buffer
	(beginning-of-line)
	(setq string 
	      (buffer-substring (point) (progn (end-of-line) (point))))
	(string-match
	 "\\([A-Z][A-Z][A-Z][0-9][0-9][0-9][0-9][0-9]\\):\\(.*\\)$" string)
	(concat rtca-quote-before-handle
		(rtca-matching-substiring string 2)
		rtca-quote-after-handle )))))

;;;
;;; change rt window format
;;;
(defun rtca-change-format (n)
  (interactive "NChange format to:")
  (cond 
   ((< n 0 )(setq rtca-format (1- rtca-format )))
   ((> n 0 )(setq rtca-format n ))
   (t (setq rtca-format (1+ rtca-format )))
   )
  (if (> rtca-format 6) (setq rtca-format 6))
  (if (< rtca-format 1) (setq rtca-format 1))
  (message (concat "Format : " (cadr (assoc rtca-format rtca-format-list))))
  )

(defun rtca-change-format-next ()
  (interactive)
  (rtca-change-format 0)
  )

(defun rtca-change-format-prev ()
  (interactive)
  (rtca-change-format -1)
  )
;;; End of change rt window format

;;;
;;; insert item
;;;
(defun rtca-insert-item (key)
  (interactive "sItem KEY : ")
  (let (string)
    (while (equal key "")
      (rtca-show-item)
      (setq key (read-string "KEY : "))
      (rtca-unique-handle))
    (if (setq string (cadr (assoc key rtca-item-list)))
	(rtca-insert-string-to-buffer 
	 rtca-input-buffer
	 string))))

(defun rtca-insert-item-0 ()
  (interactive)
  (rtca-insert-item "0"))
(defun rtca-insert-item-1 ()
  (interactive)
  (rtca-insert-item "1"))
(defun rtca-insert-item-2 ()
  (interactive)
  (rtca-insert-item "2"))
(defun rtca-insert-item-3 ()
  (interactive)
  (rtca-insert-item "3"))
(defun rtca-insert-item-4 ()
  (interactive)
  (rtca-insert-item "4"))
(defun rtca-insert-item-5 ()
  (interactive)
  (rtca-insert-item "5"))
(defun rtca-insert-item-6 ()
  (interactive)
  (rtca-insert-item "6"))
(defun rtca-insert-item-7 ()
  (interactive)
  (rtca-insert-item "7"))
(defun rtca-insert-item-8 ()
  (interactive)
  (rtca-insert-item "8"))
(defun rtca-insert-item-9 ()
  (interactive)
  (rtca-insert-item "9"))

;;;
;;; add item
;;;
(defun rtca-add-item (item key)
  (interactive "sITEM : \nsKEY : ")
  (or (equal item "")
      (progn 
	(while (equal key "")
	  (rtca-show-item)
	  (setq key (read-string "KEY : "))
	  (rtca-unique-handle))
	(setq rtca-item-list (cons (list key item) rtca-item-list))
	(message "Add item %S in key %S" item key))))

(defun rtca-add-item-from-handle ()
  (interactive)
  (let (item key)
    (setq item (read-string "ITEM : " (rtca-quote-handle)))
    (setq key (read-string "KEY : "))
    (rtca-add-item item key)))

;;;
;;; show item
;;;
(defun rtca-show-item ()
  (interactive)
  (let (key item)
    (setq rtca-tmp-list '())
    (save-selected-window 
      (select-window rtca-id-window)
      (save-current-buffer
	(set-buffer rtca-id-buffer)
	(erase-buffer)
	(save-current-buffer
	  (while (> (safe-length rtca-item-list) 0)
	    (progn
	      (setq key (caar rtca-item-list))
	      (setq item (cadr (assoc key rtca-item-list)))
	      (setq rtca-item-list (cdr rtca-item-list))
	      (unless (assoc key rtca-tmp-list)
		(setq rtca-tmp-list (cons (list key item) rtca-tmp-list))
		(rtca-add-id-buffer 
		 (concat "  "key ":" item rtca-next-line))
		)))
	  (setq rtca-item-list (nreverse rtca-tmp-list)))
	(goto-char (point-min))
	(recenter 1)
	))))



;;;END of iterm

(defun rtca-version ()
  (interactive)
  (message "Real Time Chat Adapter for NIFTY %s" rtca-version))


(run-hooks 'rtca-misc-load-hook)
;;; rtca-defun.el ends here
