;;; rtca-mode-set.el --- mode setting for rtca.
;;;	$Id: rtca-mode.el,v 1.11 2002/04/19 23:23:07 wackie Exp $	

;; Copyright (C) 2002 by Takashi KIMURA

;; Author: Takashi KIMURA <wackie@mbb.nifty.ne.jp>
;; Keywords: network, NIFTY, RTC

;; This file is free software; you can redistribute it and/or modify
;; it under the terms of the GNU General Public License as published by
;; the Free Software Foundation; either version 2, or (at your option)
;; any later version.

;; This file is distributed in the hope that it will be useful,
;; but WITHOUT ANY WARRANTY; without even the implied warranty of
;; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;; GNU General Public License for more details.

;; You should have received a copy of the GNU General Public License
;; along with GNU Emacs; see the file COPYING.  If not, write to
;; the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
;; Boston, MA 02111-1307, USA.

;;; Commentary:

;; 

;;; Code:
(provide 'rtca-mode)

;;rtca-mode
(defvar rtca-mode-map nil)
(unless rtca-mode-map 
  (let ((map (make-sparse-keymap)))
    (define-key map "\C-c\C-r" 'rtca-connect-nif)
    (define-key map "\C-c\C-g" 'rtca-auto-goto-rtc)
    (define-key map "\C-c\C-f" 'rtca-manual-goto-rtc)
    (define-key map "\C-c\C-i" 'rtca-goto-rtca-input-window)
    (define-key map "\C-c\C-d" 'rtca-goto-rtca-id-window)
    (define-key map "\C-c\C-l" 'rtca-goto-rtca-rt-window)
    (define-key map "\C-c\C-b" 'rtca-goto-rtca-history-window)
    (define-key map "\C-c\C-o" 'rtca-open-window)
    (define-key map "\C-c\C-w" 'rtca-close-history-window)
    (define-key map "\C-c\C-q" 'rtca-end)
    (define-key map "\C-c\C-a" 'rtca-disconnect-nif)
    (define-key map "\C-c," 'rtca-change-format)
    (define-key map "\C-c>" 'rtca-change-format-next)
    (define-key map "\C-c<" 'rtca-change-format-prev)
    (define-key map "\C-c+" 'rtca-add-item)
    (define-key map "\C-c*" 'rtca-insert-item)
    (define-key map "\C-c0" 'rtca-insert-item-0)
    (define-key map "\C-c1" 'rtca-insert-item-1)
    (define-key map "\C-c2" 'rtca-insert-item-2)
    (define-key map "\C-c3" 'rtca-insert-item-3)
    (define-key map "\C-c4" 'rtca-insert-item-4)
    (define-key map "\C-c5" 'rtca-insert-item-5)
    (define-key map "\C-c6" 'rtca-insert-item-6)
    (define-key map "\C-c7" 'rtca-insert-item-7)
    (define-key map "\C-c8" 'rtca-insert-item-8)
    (define-key map "\C-c9" 'rtca-insert-item-9)
    (setq rtca-mode-map map)))

(defun rtca-mode ()
  (interactive)
  (kill-all-local-variables)
  (setq major-mode 'rtca-mode
	mode-name "Rtca")
  (use-local-map rtca-mode-map)
  (run-hooks 'rtca-mode-hook)
  )

;;rtca-input-mode
(defvar rtca-input-mode-map nil)
(unless rtca-input-mode-map 
  (let ((map (make-sparse-keymap)))
    (set-keymap-parent map rtca-mode-map)
    (define-key map "\C-c\C-m" 'rtca-skk-send-line)
    (define-key map "\C-m"     'rtca-skk-send-line)
    (define-key map "\C-c\C-y" 'rtca-send-region)
    (define-key map "\M-\C-m"  'rtca-next-line)
    (setq rtca-input-mode-map map)))

(defun rtca-input-mode ()
  (interactive)
  (setq major-mode 'rtca-input-mode
	mode-name "Rtca Input")
  (use-local-map rtca-input-mode-map)
  (run-hooks 'rtca-input-mode-hook)
  )

;;rtca-log-mode
(defvar rtca-log-mode-map nil)
(unless rtca-log-mode-map 
  (let ((map (make-sparse-keymap)))
    (set-keymap-parent map rtca-mode-map)
    (define-key map "\C-c\C-m" 'rtca-quote-line-to-input)
    (define-key map "\M-\C-m"  'rtca-quote-line-to-input)
    (define-key map "\C-m"     'rtca-quote-line-and-goto-input)
    (setq rtca-log-mode-map map)))

(defun rtca-log-mode ()
  (interactive)
  (setq major-mode 'rtca-log-mode
	mode-name "Rtca LOG")
  (use-local-map rtca-log-mode-map)
  (run-hooks 'rtca-log-mode-hook)
  )

;;rtca-id-mode
(defvar rtca-id-mode-map nil)
(unless rtca-id-mode-map 
  (let ((map (make-sparse-keymap)))
    (set-keymap-parent map rtca-mode-map)
    (define-key map "\C-c\C-m" 'rtca-quote-handle-input)
    (define-key map "\M-\C-m"  'rtca-quote-handle-input)
    (define-key map "\C-m"     'rtca-quote-handle-and-goto-input)
    (define-key map "\C-cs"    'rtca-send-command)
    (define-key map "\C-cp"     'rtca-profile-command)
    (define-key map "\C-c+" 'rtca-add-item-from-handle)
    (setq rtca-id-mode-map map)))

(defun rtca-id-mode ()
  (interactive)
  (setq major-mode 'rtca-id-mode
	mode-name "Rtca ID")
  (use-local-map rtca-id-mode-map)
  (run-hooks 'rtca-id-mode-hook)
  )

;;;rtca-history-mode
(defvar rtca-history-mode-map nil)
(unless rtca-history-mode-map 
  (let ((map (make-sparse-keymap)))
    (set-keymap-parent map rtca-mode-map)
    (define-key map "\C-c\C-m" 'rtca-quote-line-to-input)
    (define-key map "\C-m"  'rtca-quote-line-and-goto-input)
    (define-key map "\M-\C-m" 'rtca-quote-line-and-goto-input-close-history)
    (setq rtca-history-mode-map map)))

(defun rtca-history-mode ()
  (interactive)
  (setq major-mode 'rtca-history-mode
	mode-name "Rtca HISTORY")
  (use-local-map rtca-history-mode-map)
  (run-hooks 'rtca-history-mode-hook)
  )

(run-hooks 'rtca-mode-load-hook)
;;; rtca-mode-set.el ends here
