;;; rtca-vars.el --- User variables for rtca.
;;;	$Id: rtca-vars.el,v 1.15 2002/04/19 13:23:43 wackie Exp $	

;; Copyright (C) 2002 by Takashi KIMURA

;; Author: Takashi KIMURA <wackie@mbb.nifty.ne.jp>
;; Keywords: network, NIFTY, RTC

;; This file is free software; you can redistribute it and/or modify
;; it under the terms of the GNU General Public License as published by
;; the Free Software Foundation; either version 2, or (at your option)
;; any later version.

;; This file is distributed in the hope that it will be useful,
;; but WITHOUT ANY WARRANTY; without even the implied warranty of
;; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;; GNU General Public License for more details.

;; You should have received a copy of the GNU General Public License
;; along with GNU Emacs; see the file COPYING.  If not, write to
;; the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
;; Boston, MA 02111-1307, USA.

;;; Commentary:

;; 

;;; Code:
(provide 'rtca-vars)

(defgroup rtca nil
  "*Real Time Chat Adapter for NIFTY"
  :prefix "rtca-"
  :group 'hypermedia)

(defcustom rtca-host "hrt.niftyserve.or.jp"
  "*$B@\B3$9$k%K%U%F%#$N%[%9%H!#(B"
  :type '(choice (const hrt.niftyserve.or.jp)
		 (const r2.niftyserve.or.jp)
		 (string :tag "$B$=$NB>(B"))
  :group 'rtca)

(defcustom rtca-auto-goto nil
  "*non-nil $B$@$H5/F0;~$K<+F0E*$K%G%U%)%k%H$N%U%)!<%i%`!"%A%c%s%M%k$K0\F0$9$k!#(B"
  :type 'boolean
  :group 'rtca)

(defcustom rtca-usr-id ""
  "*$B%K%U%F%#$N(BID $B@_Dj$7$J$$$H@\B3;~$KLd$$9g$o$;$F$/$k!#(B"
  :type '(string :tag "$B#I#D(B")
  :group 'rtca)

(defcustom rtca-handle ""
  "*RTC$B$N%O%s%I%k!#@_Dj$7$J$$$H%U%)!<%i%`$N%O%s%I%k$r$D$+$&!#(B"
  :type '(string :tag "$B%O%s%I%k(B")
  :group 'rtca)
  
(defcustom rtca-quote-mark "> "
  "*$B0zMQ$9$k$H$-$N@\F,<-!#(B"
  :type 'string
  :group 'rtca)

(defcustom rtca-quote-before-handle "$B!d(B"
  "*$B8F$S3]$1$9$k$H$-$N@\F,<-(B"
  :type 'string
  :group 'rtca)
  
(defcustom rtca-quote-after-handle "$B$5$s(B"
  "*$B8F$S3]$1$9$k$H$-$N@\Hx<-(B"
  :type 'string
  :group 'rtca)

(defcustom rtca-default-forum nil
  "*$B%G%U%)%k%H$G0\F0$9$k%U%)!<%i%`(B"
  :type 'string
  :group 'rtca)

(defcustom rtca-channel nil
  "*$B%G%U%)%k%H$G0\F0$9$k%A%c%s%M%k(B"
  :type '(choice (const :tag "$B$J$7(B" nil)
		 (const :tag "$B#1(B" 1)
		 (const :tag "$B#2(B" 2))
  :group 'rtca)

(defcustom rtca-format 4
    "*Format of RT-window"
    :type '(choice (const :tag "(Handle)" 1)
		   (const :tag "(ID)" 2)
		   (const :tag "(ID,Handle)" 3)
		   (const :tag "(Channel,Handle)" 4)
		   (const :tag "(Channel,ID)" 5)
		   (const :tag "(Channel,ID,Handle)" 6))
    :group 'rtca)

(defcustom rtca-init-file (expand-file-name "~/.rtca.el")
  "*RTCA $B$N=i4|2=%U%!%$%k(B"
  :type 'file
  :group 'rtca)


(defcustom rtca-program "telnet"
    "*Program to run to open a telnet connection."
    :type 'string
    :group 'rtca)

(defcustom rtca-unique-handle-always t
  "*non-nil $B$@$H(B ID window $B$NI=<($G!"%O%s%I%k$,ESCf$GJQ$o$k$H:FI=<($9$k(B"
  :type 'boolean
  :group 'rtca)

(defcustom rtca-input-window-height 5 
  "*input window $B$N9b$5(B"
  :type 'integer
  :group 'rtca)

(defcustom rtca-log-window-height 10 
  "*log window $B$N9b$5(B"
  :type 'integer
  :group 'rtca)

(defcustom rtca-id-unknown "XXXXXXXX"
  "*$B#I#D$,H=$i$J$$;~$NI=<((B"
  :type 'string
  :group 'rtca)

(defcustom rtca-new-line "\r"
  "*new line"
  :type 'string
  :group 'rtca)

(defcustom rtca-next-line "\n"
  "*next line"
  :type 'string
  :group 'rtca)

(defvar rtca-format-list 
  '((1 "(Handle)")
    (2 "(ID)")
    (3 "(ID,Handle)")
    (4 "(Channel,Handle)")
    (5 "(Channel,ID)")
    (6 "(Channel,ID,Handle)")))
(defvar rtca-status "OFF")
(defvar rtca-handle-list '())
(defvar rtca-history-window nil)
(defvar rtca-item-list 
  '(
    ("0" "$B!d(BALL")
    ("1" "")
    ("2" "")
    ("3" "")
    ("4" "")
    ("5" "")
    ("6" "")
    ("7" "")
    ("8" "")
    ("9" "")
))

;;; hooks
(defvar rtca-hook nil)
(defvar rtca-exit-hook nil)
(defvar rtca-mode-hook nil)
(defvar rtca-before-startup-hook nil)
(defvar rtca-exit-hook nil)
;; load hooks
(defvar rtca-vars-load-hook nil)
(defvar rtca-defun-load-hook nil)
(defvar rtca-mode-load-hook nil)
(defvar rtca-input-mode-hook nil)
(defvar rtca-log-mode-hook nil)

(run-hooks 'rtca-vars-load-hook)
;;; rtca-vars.el ends here
